//-----------------------------------------------------------------------------
// drawer.m
//
// Example of a simple drawer
//
// created by FrisbeeMonkey
// updated by Jammy
//-----------------------------------------------------------------------------

//                            USING THIS SCRIPT:
//*****************************************************************************
//  1.  Define a drawer group in your XML.
//  2.  Add your background layer to the group, something like:
//        <layer id="Drawer" image="player.Drawer" x="0" y="0" sysregion="1"/>
//        You can change any of the values but the id and sysregion
//  3.  Add a layer to pass info to this script, something like:
//        <layer id="DrawerInfo" x="13" y="0"/>
//        Set x and y to where you want your drawer to slide to
//  3.  Add any other objects(layers, buttons, sliders, etc) to your group
//  4.  Copy this script (and drawer.maki) to your scripts folder.
//  5.  If you don't have drawer.maki, compile this script.
//  6.  Add this line to the group that you want to be a drawer:
//        <script id="drawer" file="scripts/drawer.maki"/>
//  7.  Refresh your skin(F5) and try it out.
//
//  8.  Additionl by Jammy - Add a handle layer to only allow the drawer to
//      move when the handle is clicked:
//        <layer id="Drawer" image="player.Drawer" x="0" y="0" sysregion="1"/>
//        You can change any of the values but the id and sysregion
//      Note: When using multiple drawers, they must either all have handles or
//            none have handles or they may not move independantly.
//*****************************************************************************

//                               EXAMPLE XML
//=============================================================================
//<groupdef id="DrawerGroup" w="39" h="158"> <!-- 39,0 -->
//
//  <layer id="Drawer" image="player.Drawer" x="0" y="0" sysregion="1" move="0"/>
//  <layer id="DrawerInfo" x="13" y="0"/>
//  //optionally
//  <layer id="DrawerHandle" image="player.drawer.handle" x="0" y="0" sysregion="1"/>
//  //add other objects that you want to move with the drawer here
//
//  <script id="drawer" file="scripts/drawer.maki"/>
//</groupdef>
//=============================================================================
//  ***Don't forget to place your group somewhere in your layout!***
//  <group id="DrawerGroup" x="47" y="0"/>


// never forget to include std.mi
#include <lib/std.mi>

//declares global variables for use in script
Global Group DrawerGrp;
Global Layer Drawer, DrawerInfo;
//Global Button DrawerHandle;
Global GuiObject DrawerHandle;
Global Int DrawerStatus;  //0 = closed, 1 = open
Global Int StartX, StartY, EndX, EndY;

//when the script is loaded, do this
System.onScriptLoaded() {

    //gets the group that has the objects we want
    DrawerGrp = getScriptGroup();

    //now that we have the group, get the layers in the group
    Drawer = DrawerGrp.findObject("Drawer");
    DrawerInfo = DrawerGrp.findObject("DrawerInfo");
    DrawerHandle = DrawerGrp.findObject("DrawerHandle");
    
    if (DrawerHandle == Null) DrawerHandle = Drawer;

    StartX = DrawerGrp.getGuiX();
    StartY = DrawerGrp.getGuiY();
    EndX = DrawerInfo.getGuiX();
    EndY = DrawerInfo.getGuiY();

    //initialize our drawer flag to closed
    DrawerStatus = 0;

}

//when the left mouse button is released within Drawer, do this
DrawerHandle.onLeftButtonUp(int x, int y) {
  if (Drawer.isGoingToTarget()) Drawer.cancelTarget();

  if (DrawerStatus == 0) {  // if our Drawer is closed, make it slide open
      DrawerStatus = 1;  //Change the flag to Open
      DrawerGrp.setTargetX(EndX);  //set the x coordinate you want the drawer to slide to
      DrawerGrp.setTargetY(EndY);  //set the y coordinate you want the drawer to slide to
      DrawerGrp.setTargetSpeed(0.5);  //sets the speed the drawer will slide at(higher is slower)
      DrawerGrp.gotoTarget();  //now that we have our target values and speed, start moving!
  }
  else {  // if our Drawer is open, make it slide closed
      DrawerStatus = 0;  //Change the flag to Closed
      DrawerGrp.setTargetX(StartX);  //set the x coordinate you want the drawer to slide back to
      DrawerGrp.setTargetY(StartY);   //set the y coordinate you want the drawer to slide back to
      DrawerGrp.setTargetSpeed(0.5);  //sets the speed the drawer will slide at(higher is slower)
      DrawerGrp.gotoTarget();  //now that we have our target values and speed, start moving!
  }
}